from __future__ import absolute_import

import pytest

import riemann_client.client
import riemann_client.riemann_pb2
import riemann_client.transport


@pytest.fixture
def queued_client(request, string_transport):
    """A Riemann client using the StringIO transport and QueuedClient"""
    client = riemann_client.client.QueuedClient(transport=string_transport)
    client.transport.connect()

    @request.addfinalizer
    def disconnect():
        client.transport.disconnect()

    return client


@pytest.fixture
def using_simple_queue(queued_client):
    """An event queue with a single event"""
    queued_client.event(service='test')


@pytest.fixture
def large_queue(queued_client):
    """An event queue with 100 events"""
    items = ['-->{0}<--'.format(i) for i in range(0, 1000)]
    for description in items:
        queued_client.event(service='queue', description=description)
    return items


def test_simple_queue_event(queued_client, using_simple_queue):
    assert queued_client.queue.events[0].service == 'test'


def test_simple_queue_length(queued_client, using_simple_queue):
    assert len(queued_client.queue.events) == 1


def test_simple_queue_event_not_sent(queued_client, using_simple_queue):
    assert "test" not in queued_client.transport.string.getvalue()


def test_simple_queue_event_sent(queued_client, using_simple_queue):
    queued_client.flush()
    assert "test" in queued_client.transport.string.getvalue()


def test_deciqueue_length(queued_client, large_queue):
    assert len(queued_client.queue.events) == len(large_queue)


def test_deciqueue_output(queued_client, large_queue):
    queued_client.flush()
    for description in large_queue:
        assert description in queued_client.transport.string.getvalue()


def test_deciqueue_flush(queued_client, large_queue):
    queued_client.flush()
    assert len(queued_client.queue.events) == 0


def test_clear_queue(queued_client, using_simple_queue):
    queued_client.clear_queue()
    assert len(queued_client.queue.events) == 0



from __future__ import absolute_import

import sys

import pytest

import riemann_client.transport

if sys.version_info >= (3,):
    from io import StringIO as StringIO
else:
    from StringIO import StringIO


class StringTransport(riemann_client.transport.Transport):
    def connect(self):
        self.string = StringIO()

    def send(self, message):
        self.string.write(str(message.SerializeToString()))
        message.ok = True
        return message

    def disconnect(self):
        self.string.close()


@pytest.fixture
def string_transport():
    return StringTransport()



"""Wraps the riemann_py2_pb2 and riemann_py3_pb2 modules"""

import sys

__all__ = ['Event', 'Msg', 'Query', 'Attribute']

if sys.version_info >= (3,):
    from .riemann_py3_pb2 import (Event, Msg, Query, Attribute)
else:
    from .riemann_py2_pb2 import (Event, Msg, Query, Attribute)



"""Clients manage the main user facing API, and provide functions for sending
events and querying the Riemann server. UDP, TCP and TLS transports are
provided by the :py:mod:`riemann_client.transport` module, and the protocol
buffer objects are provided by the :py:mod:`riemann_client.riemann_pb2` module.
"""

from __future__ import absolute_import

import logging
try:
    from logging import NullHandler
except ImportError:
    # Create a NullHandler class in logging for python 2.6
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

import socket
try:
    from threading import RLock
    from threading import Timer
except ImportError:
    RLock = None
    Timer = None
import time

from . import riemann_pb2
from .transport import UDPTransport, TCPTransport

logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())


class Client(object):
    """A client for sending events and querying a Riemann server.

    Two sets of methods are provided - an API dealing directly with protocol
    buffer objects and an extended API that takes and returns dictionaries
    representing events.

    Protocol buffer API:

        - :py:meth:`.send_event`
        - :py:meth:`.send_events`
        - :py:meth:`.send_query`

    Extended API:

        - :py:meth:`.event`
        - :py:meth:`.events`
        - :py:meth:`.query`

    Clients do not directly manage connections to a Riemann server - these are
    managed by :py:class:`riemann_client.transport.Transport` instances, which
    provide methods to read and write messages to the server. Client instances
    can be used as a context manager, and will connect and disconnect the
    transport when entering and exiting the context.

        >>> with Client(transport) as client:
        ...     # Calls transport.connect()
        ...     client.query('true')
        ...     # Calls transport.disconnect()
    """

    def __init__(self, transport=None):
        if transport is None:
            transport = TCPTransport()
        self.transport = transport

    def __enter__(self):
        self.transport.connect()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.transport.disconnect()

    @staticmethod
    def create_event(data):
        """Translates a dictionary of event attributes to an Event object

        :param dict data: The attributes to be set on the event
        :returns: A protocol buffer ``Event`` object
        """
        event = riemann_pb2.Event()
        event.host = socket.gethostname()
        event.tags.extend(data.pop('tags', []))

        for key, value in data.pop('attributes', {}).items():
            attribute = event.attributes.add()
            attribute.key, attribute.value = key, value

        for name, value in data.items():
            if value is not None:
                setattr(event, name, value)
        return event

    def send_events(self, events):
        """Sends multiple events to Riemann in a single message

        :param events: A list or iterable of ``Event`` objects
        :returns: The response message from Riemann
        """
        message = riemann_pb2.Msg()
        for event in events:
            message.events.add().MergeFrom(event)
        return self.transport.send(message)

    def send_event(self, event):
        """Sends a single event to Riemann

        :param event: An ``Event`` protocol buffer object
        :returns: The response message from Riemann
        """
        return self.send_events((event,))

    def events(self, *events):
        """Sends multiple events in a single message

        >>> client.events({'service': 'riemann-client', 'state': 'awesome'})

         :param events: event dictionaries for :py:func:`create_event`
         :returns: The response message from Riemann
        """
        return self.send_events(self.create_event(e) for e in events)

    def event(self, **data):
        """Sends an event, using keyword arguments to create an Event

        >>> client.event(service='riemann-client', state='awesome')

        :param data: keyword arguments used for :py:func:`create_event`
        :returns: The response message from Riemann
        """
        return self.send_event(self.create_event(data))

    @staticmethod
    def create_dict(event):
        """Translates an Event object to a dictionary of event attributes

        All attributes are included, so ``create_dict(create_event(input))``
        may return more attributes than were present in the input.

        :param event: A protocol buffer ``Event`` object
        :returns: A dictionary of event attributes
        """

        data = dict()

        for descriptor, value in event.ListFields():
            if descriptor.name == 'tags':
                value = list(value)
            elif descriptor.name == 'attributes':
                value = dict(((a.key, a.value) for a in value))
            data[descriptor.name] = value

        return data

    def send_query(self, query):
        """Sends a query to the Riemann server

        :returns: The response message from Riemann
        """
        message = riemann_pb2.Msg()
        message.query.string = query
        return self.transport.send(message)

    def query(self, query):
        """Sends a query to the Riemann server

        >>> client.query('true')

        :returns: A list of event dictionaries taken from the response
        :raises Exception: if used with a :py:class:`.UDPTransport`
        """
        if isinstance(self.transport, UDPTransport):
            raise Exception('Cannot query the Riemann server over UDP')
        response = self.send_query(query)
        return [self.create_dict(e) for e in response.events]


class QueuedClient(Client):
    """A Riemann client using a queue that can be used to batch send events.

    A message object is used as a queue, with the :py:meth:`.send_event` and
    :py:meth:`.send_events` methods adding new events to the message and the
    :py:meth:`.flush` sending the message.
    """

    def __init__(self, transport=None):
        super(QueuedClient, self).__init__(transport)
        self.clear_queue()

    def flush(self):
        """Sends the waiting message to Riemann

        :returns: The response message from Riemann
        """
        response = self.transport.send(self.queue)
        self.clear_queue()
        return response

    def send_event(self, event):
        """Adds a single event to the queued message

        :returns: None - nothing has been sent to the Riemann server yet
        """
        self.send_events((event,))
        return None

    def send_events(self, events):
        """Adds multiple events to the queued message

        :returns: None - nothing has been sent to the Riemann server yet
        """
        for event in events:
            self.queue.events.add().MergeFrom(event)
        return None

    def clear_queue(self):
        """Resets the message/queue to a blank :py:class:`.Msg` object"""
        self.queue = riemann_pb2.Msg()


if RLock and Timer:  # noqa
    class AutoFlushingQueuedClient(QueuedClient):
        """A Riemann client using a queue and a timer that will automatically
        flush its contents if either:
            - the queue size exceeds :param max_batch_size: or
            - more than :param max_delay: has elapsed since the last flush and
             the queue is non-empty.

        if :param stay_connected: is False, then the transport will be
        disconnected after each flush and reconnected at the beginning of
        the next flush.
        if :param clear_on_fail: is True, then the client will discard its
        buffer after the second retry in the event of a socket error.

        A message object is used as a queue, and the following methods are
        given:
            - :py:meth:`.send_event` - add a new event to the queue
            - :py:meth:`.send_events` add a tuple of new events to the queue
            - :py:meth:`.event` - add a new event to the queue from
               keyword arguments
            - :py:meth:`.events` - add new events to the queue from
               dictionaries
            - :py:meth:`.flush` - manually force flush the queue to the
               transport
        """

        def __init__(self, transport, max_delay=0.5, max_batch_size=100,
                     stay_connected=False, clear_on_fail=False):
            super(AutoFlushingQueuedClient, self).__init__(transport)
            self.stay_connected = stay_connected
            self.clear_on_fail = clear_on_fail
            self.max_delay = max_delay
            self.max_batch_size = max_batch_size
            self.lock = RLock()
            self.event_counter = 0
            self.last_flush = time.time()
            self.timer = None

            # start the timer
            self.start_timer()

        def connect(self):
            """Connect the transport if it is not already connected."""
            if not self.is_connected():
                self.transport.connect()

        def is_connected(self):
            """Check whether the transport is connected."""
            try:
                # this will throw an exception whenever socket isn't connected
                self.transport.socket.type
                return True
            except (AttributeError, RuntimeError, socket.error):
                return False

        def event(self, **data):
            """Enqueues an event, using keyword arguments to create an Event

            >>> client.event(service='riemann-client', state='awesome')

            :param data: keyword arguments used for :py:func:`create_event`
            """
            self.send_events((self.create_event(data),))

        def events(self, *events):
            """Enqueues multiple events in a single message

            >>> client.events({'service': 'riemann-client',
            >>>                'state': 'awesome'})

             :param events: event dictionaries for :py:func:`create_event`
             :returns: The response message from Riemann
            """
            self.send_events(self.create_event(evd) for evd in events)

        def send_events(self, events):
            """Enqueues multiple events

            :param events: A list or iterable of ``Event`` objects
            :returns: The response message from Riemann
            """
            with self.lock:
                for event in events:
                    self.queue.events.add().MergeFrom(event)
                    self.event_counter += 1
                    self.check_for_flush()

        def flush(self):
            """Sends the events in the queue to Riemann in a single protobuf
            message

            :returns: The response message from Riemann
            """
            response = None
            with self.lock:
                if not self.is_connected():
                    self.connect()
                try:
                    response = super(AutoFlushingQueuedClient, self).flush()
                except socket.error:
                    # log and retry
                    logger.warning("Socket error on flushing. "
                                   "Attempting reconnect and retry...")
                    try:
                        self.transport.disconnect()
                        self.connect()
                        response = (
                            super(AutoFlushingQueuedClient, self).flush())
                    except Exception:
                        logger.warning("Socket error on flushing "
                                       "second attempt. Batch discarded.")
                        self.transport.disconnect()
                        if self.clear_on_fail:
                            self.clear_queue()
                self.event_counter = 0
                if not self.stay_connected:
                    self.transport.disconnect()
                self.last_flush = time.time()
            self.start_timer()
            return response

        def check_for_flush(self):
            """Checks the conditions for flushing the queue"""
            if (self.event_counter >= self.max_batch_size or
                    (time.time() - self.last_flush) >= self.max_delay):
                self.flush()

        def start_timer(self):
            """Cycle the timer responsible for periodically flushing the queue
            """
            if self.timer:
                self.timer.cancel()
            self.timer = Timer(self.max_delay, self.check_for_flush)
            self.timer.daemon = True
            self.timer.start()

        def stop_timer(self):
            """Stops the current timer

            a :py:meth:`.flush` event will reactviate the timer
            """
            self.timer.cancel()


__all__ = 'Client', 'QueuedClient', 'AutoFlushingQueuedClient'



"""A Python Riemann client and command line tool"""

from .client import Client, QueuedClient
from .transport import (
    BlankTransport,
    RiemannError,
    SocketTransport,
    TCPTransport,
    TLSTransport,
    UDPTransport,
)

try:
    from .client import AutoFlushingQueuedClient
except ImportError:
    pass

__version__ = '6.4.0'
__author__ = 'Sam Clements <sam.clements@datasift.com>'

__all__ = (
    "AutoFlushingQueuedClient",
    "BlankTransport",
    "Client",
    "QueuedClient",
    "RiemannError",
    "SocketTransport",
    "TCPTransport",
    "TLSTransport",
    "UDPTransport",
)



#!/usr/bin/env python2.6

import sys

import setuptools

if sys.version_info >= (3,):
    protobuf = 'protobuf>=3.2.0,<4.0.0'
else:
    protobuf = 'protobuf>=2.3.0,<4.0.0'

setuptools.setup(
    name='riemann-client',
    version='7.0.0pre1',

    author="Sam Clements",
    author_email="sam.clements@datasift.com",

    url="https://github.com/borntyping/python-riemann-client",
    description="A Riemann client and command line tool",
    long_description=open('README.rst').read(),
    license="MIT",

    packages=[
        'riemann_client',
    ],

    install_requires=[
        'click>=3.1',
        protobuf
    ],

    extras_require={
        'docs': [
            'sphinx',
            'sphinx_rtd_theme'
        ]
    },

    entry_points={
        'console_scripts': [
            'riemann-client = riemann_client.command:main',
        ]
    },

    classifiers=[
        'Development Status :: 7 - Inactive',
        'License :: OSI Approved',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries',
        'Topic :: System :: Monitoring',
        'Topic :: System :: Networking',
        'Topic :: System :: Systems Administration'
    ],
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: riemann.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='riemann.proto',
  package='riemann_client',
  syntax='proto2',
  serialized_pb=_b('\n\rriemann.proto\x12\x0eriemann_client\"\x81\x01\n\x05State\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\r\n\x05state\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\t\x12\x0c\n\x04host\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x0c\n\x04once\x18\x06 \x01(\x08\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x0b\n\x03ttl\x18\x08 \x01(\x02\"\xdd\x01\n\x05\x45vent\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\r\n\x05state\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\t\x12\x0c\n\x04host\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x0c\n\x04tags\x18\x07 \x03(\t\x12\x0b\n\x03ttl\x18\x08 \x01(\x02\x12-\n\nattributes\x18\t \x03(\x0b\x32\x19.riemann_client.Attribute\x12\x15\n\rmetric_sint64\x18\r \x01(\x12\x12\x10\n\x08metric_d\x18\x0e \x01(\x01\x12\x10\n\x08metric_f\x18\x0f \x01(\x02\"\x17\n\x05Query\x12\x0e\n\x06string\x18\x01 \x01(\t\"\x94\x01\n\x03Msg\x12\n\n\x02ok\x18\x02 \x01(\x08\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12%\n\x06states\x18\x04 \x03(\x0b\x32\x15.riemann_client.State\x12$\n\x05query\x18\x05 \x01(\x0b\x32\x15.riemann_client.Query\x12%\n\x06\x65vents\x18\x06 \x03(\x0b\x32\x15.riemann_client.Event\"\'\n\tAttribute\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x01(\tB\x1a\n\x11\x63om.aphyr.riemannB\x05Proto')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_STATE = _descriptor.Descriptor(
  name='State',
  full_name='riemann_client.State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='riemann_client.State.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='riemann_client.State.state', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='service', full_name='riemann_client.State.service', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='host', full_name='riemann_client.State.host', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='riemann_client.State.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='once', full_name='riemann_client.State.once', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='riemann_client.State.tags', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ttl', full_name='riemann_client.State.ttl', index=7,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=163,
)


_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='riemann_client.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='riemann_client.Event.time', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='riemann_client.Event.state', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='service', full_name='riemann_client.Event.service', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='host', full_name='riemann_client.Event.host', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='description', full_name='riemann_client.Event.description', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tags', full_name='riemann_client.Event.tags', index=5,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ttl', full_name='riemann_client.Event.ttl', index=6,
      number=8, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='riemann_client.Event.attributes', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_sint64', full_name='riemann_client.Event.metric_sint64', index=8,
      number=13, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_d', full_name='riemann_client.Event.metric_d', index=9,
      number=14, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='metric_f', full_name='riemann_client.Event.metric_f', index=10,
      number=15, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=166,
  serialized_end=387,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='riemann_client.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string', full_name='riemann_client.Query.string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=389,
  serialized_end=412,
)


_MSG = _descriptor.Descriptor(
  name='Msg',
  full_name='riemann_client.Msg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ok', full_name='riemann_client.Msg.ok', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error', full_name='riemann_client.Msg.error', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='states', full_name='riemann_client.Msg.states', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='query', full_name='riemann_client.Msg.query', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='events', full_name='riemann_client.Msg.events', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=415,
  serialized_end=563,
)


_ATTRIBUTE = _descriptor.Descriptor(
  name='Attribute',
  full_name='riemann_client.Attribute',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='riemann_client.Attribute.key', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='riemann_client.Attribute.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=565,
  serialized_end=604,
)

_EVENT.fields_by_name['attributes'].message_type = _ATTRIBUTE
_MSG.fields_by_name['states'].message_type = _STATE
_MSG.fields_by_name['query'].message_type = _QUERY
_MSG.fields_by_name['events'].message_type = _EVENT
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['Msg'] = _MSG
DESCRIPTOR.message_types_by_name['Attribute'] = _ATTRIBUTE

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), dict(
  DESCRIPTOR = _STATE,
  __module__ = 'riemann_pb2'
  # @@protoc_insertion_point(class_scope:riemann_client.State)
  ))
_sym_db.RegisterMessage(State)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), dict(
  DESCRIPTOR = _EVENT,
  __module__ = 'riemann_pb2'
  # @@protoc_insertion_point(class_scope:riemann_client.Event)
  ))
_sym_db.RegisterMessage(Event)

Query = _reflection.GeneratedProtocolMessageType('Query', (_message.Message,), dict(
  DESCRIPTOR = _QUERY,
  __module__ = 'riemann_pb2'
  # @@protoc_insertion_point(class_scope:riemann_client.Query)
  ))
_sym_db.RegisterMessage(Query)

Msg = _reflection.GeneratedProtocolMessageType('Msg', (_message.Message,), dict(
  DESCRIPTOR = _MSG,
  __module__ = 'riemann_pb2'
  # @@protoc_insertion_point(class_scope:riemann_client.Msg)
  ))
_sym_db.RegisterMessage(Msg)

Attribute = _reflection.GeneratedProtocolMessageType('Attribute', (_message.Message,), dict(
  DESCRIPTOR = _ATTRIBUTE,
  __module__ = 'riemann_pb2'
  # @@protoc_insertion_point(class_scope:riemann_client.Attribute)
  ))
_sym_db.RegisterMessage(Attribute)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\021com.aphyr.riemannB\005Proto'))
# @@protoc_insertion_point(module_scope)
